@@ -1,4 +1,5 @@
-from typing import Callable, Optional
+from logging import Logger
+from typing import Callable, Optional, Union
 
 from pydantic import BaseModel
 
@@ -7,10 +8,15 @@ from multibotkit.states.managers.memory import MemoryStateManager
 
 
 class BaseDispatcher:
-    def __init__(self, state_manager: BaseStateManager = MemoryStateManager()):
+    def __init__(
+        self,
+        state_manager: BaseStateManager = MemoryStateManager(),
+        logger: Optional[Union[Logger, Callable]] = None
+    ):
         self._handlers = []
         self._default_handler = None
         self.state_manager = state_manager
+        self.logger = logger
 
     def handler(self, func=None, state_object_func=None):
         def wrapper(f):
