@@ -10,15 +10,19 @@ from tenacity import (
     stop_after_attempt,
     wait_exponential,
 )
-
 from multibotkit.helpers.base_helper import BaseHelper
 from multibotkit.schemas.telegram.outgoing import (
+    Document,
+    EditMessageMediaModel,
     InlineKeyboardMarkup,
+    InputMedia,
+    InputMediaPhoto,
     Message,
     Photo,
     ReplyKeyboardMarkup,
     SetWebhookParams,
-    WebhookInfo, InputMediaPhoto, MediaGroup,
+    WebhookInfo,
+    MediaGroup,
 )
 
 
@@ -184,6 +188,210 @@ class TelegramHelper(BaseHelper):
             data = {"chat_id": chat_id, "message_id": message_id, "reply_markup": {}}
         r = await self._perform_async_request(url, data)
         return r
+    
+    def sync_edit_message_media(
+        self,
+        media: Union[str, IO],
+        media_type: str,
+        caption: Optional[str] = None,
+        reply_markup: Optional[InlineKeyboardMarkup] = None,
+        chat_id: Optional[int] = None,
+        message_id: Optional[int] = None,
+        inline_message_id: Optional[Union[int, str]] = None
+    ):
+        if type(media) == str:
+            if media.startswith("http://") or media.startswith("https://"):
+                media_obj = InputMedia(
+                    type=media_type,
+                    media=media,
+                    caption=caption
+                )
+                data_obj = EditMessageMediaModel(
+                    chat_id=chat_id,
+                    message_id=message_id,
+                    inline_message_id=inline_message_id,
+                    media=media_obj,
+                    reply_markup=reply_markup
+                )
+
+                url = self.tg_base_url + "editMessageMedia"
+                data = data_obj.dict(exclude_none=True)
+
+                r = self._perform_sync_request(url, data)
+                return r
+            
+            ends = [".jpg", ".jpeg", ".gif", ".png"]
+            for end in ends:
+                if media.endswith(end):
+                    opened_media = open(media, "rb")
+
+                    media_obj = InputMedia(
+                        type=media_type,
+                        media=f"attach://{media}",
+                        caption=caption
+                    )
+                    data_obj = EditMessageMediaModel(
+                        chat_id=chat_id,
+                        message_id=message_id,
+                        inline_message_id=inline_message_id,
+                        media=media_obj,
+                        reply_markup=reply_markup
+                    )
+
+                    url = self.tg_base_url + "editMessageMedia"
+                    data = data_obj.dict(exclude_none=True)
+                    data["media"] = json.dumps(data["media"])
+                    if "reply_markup" in data.keys():
+                        data["reply_markup"] = json.dumps(data["reply_markup"])
+                    files = {media: opened_media}
+
+                    r = self._perform_sync_request(url, data, use_json=False, files=files)
+                    return r
+            
+            media_obj = InputMedia(
+                type=media_type,
+                media=media,
+                caption=caption
+            )
+            data_obj = EditMessageMediaModel(
+                chat_id=chat_id,
+                message_id=message_id,
+                inline_message_id=inline_message_id,
+                media=media_obj,
+                reply_markup=reply_markup
+            )
+
+            url = self.tg_base_url + "editMessageMedia"
+            data = data_obj.dict(exclude_none=True)
+
+            r = self._perform_sync_request(url, data)
+            return r
+        
+        media_obj = InputMedia(
+            type=media_type,
+            media="attach://media",
+            caption=caption
+        )
+        data_obj = EditMessageMediaModel(
+            chat_id=chat_id,
+            message_id=message_id,
+            inline_message_id=inline_message_id,
+            media=media_obj,
+            reply_markup=reply_markup
+        )
+
+        url = self.tg_base_url + "editMessageMedia"
+        data = data_obj.dict(exclude_none=True)
+        data["media"] = json.dumps(data["media"])
+        if "reply_markup" in data.keys():
+            data["reply_markup"] = json.dumps(data["reply_markup"])
+        files = {"media": media}
+
+        r = self._perform_sync_request(url, data, use_json=False, files=files)
+        return r
+    
+
+    async def async_edit_message_media(
+        self,
+        media: Union[str, IO],
+        media_type: str,
+        caption: Optional[str] = None,
+        reply_markup: Optional[InlineKeyboardMarkup] = None,
+        chat_id: Optional[int] = None,
+        message_id: Optional[int] = None,
+        inline_message_id: Optional[Union[int, str]] = None
+    ):
+        if type(media) == str:
+            if media.startswith("http://") or media.startswith("https://"):
+                media_obj = InputMedia(
+                    type=media_type,
+                    media=media,
+                    caption=caption
+                )
+                data_obj = EditMessageMediaModel(
+                    chat_id=chat_id,
+                    message_id=message_id,
+                    inline_message_id=inline_message_id,
+                    media=media_obj,
+                    reply_markup=reply_markup
+                )
+
+                url = self.tg_base_url + "editMessageMedia"
+                data = data_obj.dict(exclude_none=True)
+
+                r = await self._perform_async_request(url, data)
+                return r
+            
+            ends = [".jpg", ".jpeg", ".gif", ".png"]
+            for end in ends:
+                if media.endswith(end):
+                    opened_media = open(media, "rb")
+
+                    media_obj = InputMedia(
+                        type=media_type,
+                        media=f"attach://{media}",
+                        caption=caption
+                    )
+                    data_obj = EditMessageMediaModel(
+                        chat_id=chat_id,
+                        message_id=message_id,
+                        inline_message_id=inline_message_id,
+                        media=media_obj,
+                        reply_markup=reply_markup
+                    )
+
+                    url = self.tg_base_url + "editMessageMedia"
+                    data = data_obj.dict(exclude_none=True)
+                    data["media"] = json.dumps(data["media"])
+                    if "reply_markup" in data.keys():
+                        data["reply_markup"] = json.dumps(data["reply_markup"])
+                    files = {media: opened_media}
+
+                    r = await self._perform_async_request(url, data, use_json=False, files=files)
+                    return r
+            
+            media_obj = InputMedia(
+                type=media_type,
+                media=media,
+                caption=caption
+            )
+            data_obj = EditMessageMediaModel(
+                chat_id=chat_id,
+                message_id=message_id,
+                inline_message_id=inline_message_id,
+                media=media_obj,
+                reply_markup=reply_markup
+            )
+
+            url = self.tg_base_url + "editMessageMedia"
+            data = data_obj.dict(exclude_none=True)
+
+            r = await self._perform_async_request(url, data)
+            return r
+        
+        media_obj = InputMedia(
+            type=media_type,
+            media="attach://media",
+            caption=caption
+        )
+        data_obj = EditMessageMediaModel(
+            chat_id=chat_id,
+            message_id=message_id,
+            inline_message_id=inline_message_id,
+            media=media_obj,
+            reply_markup=reply_markup
+        )
+
+        url = self.tg_base_url + "editMessageMedia"
+        data = data_obj.dict(exclude_none=True)
+        data["media"] = json.dumps(data["media"])
+        if "reply_markup" in data.keys():
+            data["reply_markup"] = json.dumps(data["reply_markup"])
+        files = {"media": media}
+
+        r = await self._perform_async_request(url, data, use_json=False, files=files)
+        return r
+
 
     def sync_send_photo(
         self,
@@ -235,6 +443,8 @@ class TelegramHelper(BaseHelper):
 
                     url = self.tg_base_url + "sendPhoto"
                     data = photo_obj.dict(exclude_none=True)
+                    if "reply_markup" in data.keys():
+                        data["reply_markup"] = json.dumps(data["reply_markup"])
                     files = {photo: opened_photo}
                     
                     r = self._perform_sync_request(url, data, use_json=False, files=files)
@@ -272,6 +482,8 @@ class TelegramHelper(BaseHelper):
 
         url = self.tg_base_url + "sendPhoto"
         data = photo_obj.dict(exclude_none=True)
+        if "reply_markup" in data.keys():
+            data["reply_markup"] = json.dumps(data["reply_markup"])
         files = {"image": photo}
         
         r = self._perform_sync_request(url, data, use_json=False, files=files)
@@ -305,7 +517,7 @@ class TelegramHelper(BaseHelper):
 
                 url = self.tg_base_url + "sendPhoto"
                 data = photo_obj.dict(exclude_none=True)
-                r = self._perform_sync_request(url, data)
+                r = await self._perform_async_request(url, data)
                 return r
             
             ends = [".jpg", ".jpeg", ".gif", ".png"]
@@ -327,8 +539,10 @@ class TelegramHelper(BaseHelper):
 
                     url = self.tg_base_url + "sendPhoto"
                     data = photo_obj.dict(exclude_none=True)
+                    if "reply_markup" in data.keys():
+                        data["reply_markup"] = json.dumps(data["reply_markup"])
                     files = {photo: content}
-                    r = self._perform_sync_request(url, data, use_json=False, files=files)
+                    r = await self._perform_async_request(url, data, use_json=False, files=files)
                     return r
             
             photo_obj = Photo(
@@ -345,7 +559,7 @@ class TelegramHelper(BaseHelper):
 
             url = self.tg_base_url + "sendPhoto"
             data = photo_obj.dict(exclude_none=True)
-            r = self._perform_sync_request(url, data)
+            r = await self._perform_async_request(url, data)
             return r
         
         photo_obj = Photo(
@@ -362,8 +576,118 @@ class TelegramHelper(BaseHelper):
 
         url = self.tg_base_url + "sendPhoto"
         data = photo_obj.dict(exclude_none=True)
+        if "reply_markup" in data.keys():
+            data["reply_markup"] = json.dumps(data["reply_markup"])
         files = {"image": photo}
-        r = self._perform_sync_request(url, data, use_json=False, files=files)
+        r = await self._perform_async_request(url, data, use_json=False, files=files)
+        return r
+    
+
+    def sync_send_document(
+        self,
+        chat_id: int,
+        document: Union[str, IO],
+        caption: Optional[str] = None,
+        file_name: Optional[str] = None,
+        parse_mode: str = "HTML",
+        disable_notification: Optional[bool] = None,
+        protect_content: Optional[bool] = None,
+        reply_to_message_id: Optional[int] = None,
+        allow_sending_without_reply: Optional[bool] = None,
+        reply_markup: Optional[Union[InlineKeyboardMarkup, ReplyKeyboardMarkup]] = None
+    ):
+        files = {}
+        document_str = None
+
+        if type(document) == str:
+            if not (document.startswith("http://") or document.startswith("https://")):
+                ends = [".pdf", ".doc", ".docx", ".xls", ".xlsx", ".ppt", ".pptx"]
+                for end in ends:
+                    if document.endswith(end):
+                        files["document"] = (file_name if file_name else document, open(document, "rb"))
+                        document_str = "attach://document"
+            if document_str is None:
+                document_str = document
+        else:
+            document_str = "attach://document"
+            files["document"] = (file_name if file_name else "file", document)
+
+        document_obj = Document(
+            chat_id=chat_id,
+            document=document_str,
+            caption=caption,
+            parse_mode=parse_mode,
+            disable_notification=disable_notification,
+            protect_content=protect_content,
+            reply_to_message_id=reply_to_message_id,
+            allow_sending_without_reply=allow_sending_without_reply,
+            reply_markup=reply_markup
+        )
+
+        url = self.tg_base_url + "sendDocument"
+        data = document_obj.dict(exclude_none=True)
+        if "reply_markup" in data.keys():
+            data["reply_markup"] = json.dumps(data["reply_markup"])
+
+        if len(files.keys()):
+            r = self._perform_sync_request(url, data, use_json=False, files=files)
+            return r
+        
+        r = self._perform_sync_request(url, data)
+        return r
+
+    async def async_send_document(
+        self,
+        chat_id: int,
+        document: Union[str, IO],
+        caption: Optional[str] = None,
+        file_name: Optional[str] = None,
+        parse_mode: str = "HTML",
+        disable_notification: Optional[bool] = None,
+        protect_content: Optional[bool] = None,
+        reply_to_message_id: Optional[int] = None,
+        allow_sending_without_reply: Optional[bool] = None,
+        reply_markup: Optional[Union[InlineKeyboardMarkup, ReplyKeyboardMarkup]] = None
+    ):
+        files = {}
+        document_str = None
+        
+        if type(document) == str:
+            if not (document.startswith("http://") or document.startswith("https://")):
+                ends = [".pdf", ".doc", ".docx", ".xls", ".xlsx", ".ppt", ".pptx"]
+                for end in ends:
+                    if document.endswith(end):
+                        files["document"] = (file_name if file_name else document, open(document, "rb"))
+                        document_str = "attach://document"
+            if document_str is None:
+                document_str = document
+        else:
+            document_str = "attach://document"
+            files["document"] = (file_name if file_name else "file", document)
+
+        document_obj = Document(
+            chat_id=chat_id,
+            document=document_str,
+            caption=caption,
+            parse_mode=parse_mode,
+            disable_notification=disable_notification,
+            protect_content=protect_content,
+            reply_to_message_id=reply_to_message_id,
+            allow_sending_without_reply=allow_sending_without_reply,
+            reply_markup=reply_markup
+        )
+
+        url = self.tg_base_url + "sendDocument"
+        data = document_obj.dict(exclude_none=True)
+        
+        if "reply_markup" in data.keys():
+            data["reply_markup"] = json.dumps(data["reply_markup"])
+
+        if len(files.keys()):
+            r = await self._perform_async_request(url, data, use_json=False, files=files)
+            return r
+        
+        r = await self._perform_async_request(url, data)
         return r
 
 
